@@ -92,9 +92,11 @@ bool            exclude_ecores     = true;
 bool            smp_enabled        = true;
 
 bool            enable_big_status  = true;
-bool            enable_temperature = true;
 bool            enable_trace       = false;
 
+bool            enable_temp_cpu    = true;
+bool            enable_temp_ram    = true;              // DDR5+ temperature polling
+
 bool            enable_sm          = true;
 bool            enable_bench       = true;
 bool            enable_mch_read    = true;
@@ -103,6 +105,7 @@ bool            enable_numa        = false;
 bool            enable_ecc_polling = false;
 
 bool            pause_at_start     = true;
+bool            dark_mode          = false;
 
 power_save_t    power_save         = POWER_SAVE_HIGH;
 
@@ -110,6 +113,7 @@ bool            enable_tty         = false;
 uintptr_t       tty_address        = 0x3F8;             // Legacy IO or MMIO Address accepted
 int             tty_baud_rate      = 115200;
 int             tty_update_period  = 2;                 // Update TTY every 2 seconds (default)
+bool            tty_new_line       = false;
 
 uint32_t        tty_mmio_ref_clk   = UART_REF_CLK_MMIO; // Reference clock for MMIO (in Hz)
 int             tty_mmio_stride    = 4;                 // Stride for MMIO (register width in bytes)
@@ -206,6 +210,8 @@ static void parse_option(const char *option, const char *params)
 
     if (strncmp(option, "console", 8) == 0) {
         parse_serial_params(params);
+    } else if (strncmp(option, "newline", 7) == 0) {
+        tty_new_line = true;
     } else if (strncmp(option, "cpuseqmode", 11) == 0) {
         if (strncmp(params, "par", 4) == 0) {
             cpu_mode = PAR;
@@ -214,6 +220,8 @@ static void parse_option(const char *option, const char *params)
         } else if (strncmp(params, "rr", 3) == 0 || strncmp(params, "one", 4) == 0) {
             cpu_mode = ONE;
         }
+    } else if (strncmp(option, "dark", 5) == 0) {
+        dark_mode = true;
     } else if (strncmp(option, "reportmode", 11) == 0) {
         if (strncmp(params, "none", 5) == 0) {
             error_mode = ERROR_MODE_NONE;
@@ -223,6 +231,10 @@ static void parse_option(const char *option, const char *params)
             error_mode = ERROR_MODE_ADDRESS;
         } else if (strncmp(params, "badram", 7) == 0) {
             error_mode = ERROR_MODE_BADRAM;
+        } else if (strncmp(params, "memmap", 7) == 0) {
+            error_mode = ERROR_MODE_MEMMAP;
+        } else if (strncmp(params, "pages", 6) == 0) {
+            error_mode = ERROR_MODE_PAGES;
         }
     } else if (strncmp(option, "keyboard", 9) == 0 && params != NULL) {
         if (strncmp(params, "legacy", 7) == 0) {
@@ -262,6 +274,11 @@ static void parse_option(const char *option, const char *params)
         enable_trace = true;
     } else if (strncmp(option, "usbdebug", 9) == 0) {
         usb_init_options |= USB_DEBUG;
+        if (strncmp(params, "hub", 4) == 0) {
+            usb_init_options |= USB_DEBUG_HUB;
+        } else if (strncmp(params, "kbd", 4) == 0) {
+            usb_init_options |= USB_DEBUG_KBD;
+	}
     } else if (strncmp(option, "usbinit", 8) == 0) {
         if (strncmp(params, "1", 2) == 0) {
             usb_init_options |= USB_2_STEP_INIT;
@@ -652,7 +669,9 @@ static void error_mode_menu(void)
     prints(POP_R+4, POP_LI, "<F2>  Error summary");
     prints(POP_R+5, POP_LI, "<F3>  Individual errors");
     prints(POP_R+6, POP_LI, "<F4>  BadRAM patterns");
-    prints(POP_R+7, POP_LI, "<F10> Exit menu");
+    prints(POP_R+7, POP_LI, "<F5>  Linux memmap");
+    prints(POP_R+8, POP_LI, "<F6>  Bad pages");
+    prints(POP_R+9, POP_LI, "<F10> Exit menu");
     printc(POP_R+3+error_mode, POP_LM, '*');
 
     bool tty_update = enable_tty;
@@ -671,6 +690,8 @@ static void error_mode_menu(void)
           case '2':
           case '3':
           case '4':
+          case '5':
+          case '6':
             set_error_mode(ch - '1');
             break;
           case 'u':
@@ -679,7 +700,7 @@ static void error_mode_menu(void)
             }
             break;
           case 'd':
-            if (error_mode < 3) {
+            if (error_mode < 5) {
                 set_error_mode(error_mode + 1);
             }
             break;
@@ -882,11 +903,10 @@ void config_menu(bool initial)
             if (!smp_enabled)  set_foreground_colour(BOLD+BLACK);
             prints(POP_R+7,  POP_LI, "<F5>  CPU selection");
             if (!smp_enabled)  set_foreground_colour(WHITE);
-            //if (no_temperature) set_foreground_colour(BOLD+BLACK);
-            printf(POP_R+8,  POP_LI, "<F6>  Temperature %s", enable_temperature ? "disable" : "enable ");
-            //if (no_temperature) set_foreground_colour(WHITE);
-            printf(POP_R+9,  POP_LI, "<F7>  Boot trace %s",  enable_trace  ? "disable" : "enable ");
-            prints(POP_R+10, POP_LI, "<F10> Exit menu");
+            printf(POP_R+8,  POP_LI, "<F6>  CPU Temperature %s", enable_temp_cpu ? "disable" : "enable ");
+            printf(POP_R+9,  POP_LI, "<F7>  RAM Temperature %s", enable_temp_ram ? "disable" : "enable ");
+            printf(POP_R+10, POP_LI, "<F8>  Boot trace %s",  enable_trace  ? "disable" : "enable ");
+            prints(POP_R+11, POP_LI, "<F10> Exit menu");
         } else {
             prints(POP_R+7,  POP_LI, "<F5>  Skip current test");
             prints(POP_R+8 , POP_LI, "<F10> Exit menu");
@@ -923,10 +943,15 @@ void config_menu(bool initial)
             break;
           case '6':
             if (initial) {
-                enable_temperature = !enable_temperature;
+                enable_temp_cpu = !enable_temp_cpu;
             }
             break;
           case '7':
+            if (initial) {
+                enable_temp_ram = !enable_temp_ram;
+            }
+            break;
+          case '8':
             if (initial) {
                 enable_trace = !enable_trace;
             }
@@ -942,7 +967,7 @@ void config_menu(bool initial)
     }
 
     restore_screen_region(POP_REGION, popup_save_buffer);
-    set_background_colour(BLUE);
+    set_background_colour(palette.background);
 
     if (enable_tty) {
         tty_send_region(POP_REGION);
