@@ -26,7 +26,9 @@ typedef enum {
     ERROR_MODE_NONE,
     ERROR_MODE_SUMMARY,
     ERROR_MODE_ADDRESS,
-    ERROR_MODE_BADRAM
+    ERROR_MODE_BADRAM,
+    ERROR_MODE_MEMMAP,
+    ERROR_MODE_PAGES,
 } error_mode_t;
 
 typedef enum {
@@ -52,9 +54,11 @@ extern bool         exclude_ecores;
 extern bool         smp_enabled;
 
 extern bool         enable_big_status;
-extern bool         enable_temperature;
 extern bool         enable_trace;
 
+extern bool         enable_temp_cpu;
+extern bool         enable_temp_ram;
+
 extern bool         enable_sm;
 extern bool         enable_tty;
 extern bool         enable_bench;
@@ -63,12 +67,14 @@ extern bool         enable_ecc_polling;
 extern bool         enable_numa;
 
 extern bool         pause_at_start;
+extern bool         dark_mode;
 
 extern power_save_t power_save;
 
 extern uintptr_t    tty_address;
 extern int          tty_baud_rate;
 extern int          tty_update_period;
+extern bool         tty_new_line;
 
 extern uint32_t     tty_mmio_ref_clk;
 extern int          tty_mmio_stride;
