@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0-only
-// Copyright (C) 2020-2022 Martin Whitaker.
+// Copyright (C) 2020-2024 Martin Whitaker.
 //
 // Derived from Linux 5.6 arch/x86/boot/compressed/eboot.c and extracts
 // from drivers/firmware/efi/libstub:
@@ -16,8 +16,6 @@
 
 #include "string.h"
 
-#define DEBUG 0
-
 //------------------------------------------------------------------------------
 // Constants
 //------------------------------------------------------------------------------
@@ -37,6 +35,13 @@ static efi_guid_t EFI_LOADED_IMAGE_PROTOCOL_GUID      = { 0x5b1b31a1, 0x9562, 0x
 
 static efi_system_table_t *sys_table = NULL;
 
+static uint32_t pref_h_resolution;
+static uint32_t pref_v_resolution;
+
+static bool rotate;
+
+static bool debug;
+
 //------------------------------------------------------------------------------
 // Macro Functions
 //------------------------------------------------------------------------------
@@ -91,7 +96,6 @@ static void print_string(char *str)
     }
 }
 
-#if DEBUG
 static void print_dec(unsigned value)
 {
     char buffer[16];
@@ -142,32 +146,31 @@ static void test_frame_buffer(screen_info_t *si)
     pixel_value.word = (r_value << si->red_pos) | (g_value << si->green_pos) | (b_value << si->blue_pos);
 
     uintptr_t lfb_base = si->lfb_base;
-#ifdef __x86_64__
+#if (ARCH_BITS == 64)
     if (LFB_CAPABILITY_64BIT_BASE & si->capabilities) {
         lfb_base |= (uintptr_t)si->ext_lfb_base << 32;
     }
 #endif
 
     uint8_t *lfb_row = (uint8_t *)lfb_base;
-    for (int y = 0; y < 8; y++) {
+    for (int y = 0; y < 4; y++) {
         for (int x = 0; x < si->lfb_width; x++) {
             for (int b = 0; b < pixel_size; b++) {
                 lfb_row[x * pixel_size + b] = pixel_value.byte[b];
             }
         }
-        lfb_row += si->lfb_linelength;
+        lfb_row += si->lfb_linelength * 2;
     }
     lfb_row += (si->lfb_height - 16) * si->lfb_linelength;
-    for (int y = 0; y < 8; y++) {
+    for (int y = 0; y < 4; y++) {
         for (int x = 0; x < si->lfb_width; x++) {
             for (int b = 0; b < pixel_size; b++) {
                 lfb_row[x * pixel_size + b] = pixel_value.byte[b];
             }
         }
-        lfb_row += si->lfb_linelength;
+        lfb_row += si->lfb_linelength * 2;
     }
 }
-#endif
 
 static int get_cmd_line_length(efi_loaded_image_t *image)
 {
@@ -176,6 +179,11 @@ static int get_cmd_line_length(efi_loaded_image_t *image)
     efi_char16_t *cmd_line = (efi_char16_t *)image->load_options;
     int max_length = image->load_options_size / sizeof(efi_char16_t);
     int length = 0;
+    // Skip Unicode byte order mark if present
+    if (cmd_line[0] == u'\uFEFF') {
+        cmd_line = &cmd_line[1];
+        max_length--;
+    }
     while (length < max_length && cmd_line[length] > 0x00 && cmd_line[length] < 0x80) {
         length++;
     }
@@ -185,12 +193,93 @@ static int get_cmd_line_length(efi_loaded_image_t *image)
 static void get_cmd_line(efi_loaded_image_t *image, int num_chars, char *buffer)
 {
     efi_char16_t *cmd_line = (efi_char16_t *)image->load_options;
+    if (cmd_line[0] == u'\uFEFF') {
+        cmd_line = &cmd_line[1];
+    }
     for (int i = 0; i < num_chars; i++) {
         buffer[i] = cmd_line[i];
     }
     buffer[num_chars] = '\0';
 }
 
+static void parse_option(const char *option, int option_length)
+{
+    if ((option_length == 8) && (strncmp(option, "efidebug", 8) == 0)) {
+        debug = true;
+        return;
+    }
+
+    if ((option_length < 8) || (strncmp(option, "screen.", 7) != 0))
+        return;
+
+    option_length -= 7;
+    option += 7;
+    if ((option_length == 6) && (strncmp(option, "rhs-up", 6) == 0)) {
+        rotate = true;
+        return;
+    }
+    if ((option_length == 6) && (strncmp(option, "lhs-up", 6) == 0)) {
+        rotate = true;
+        return;
+    }
+    if ((option_length >= 6) && (strncmp(option, "mode=", 5) == 0)) {
+        option_length -= 5;
+        option += 5;
+        if ((option_length == 4) && (strncmp(option, "bios", 4) == 0)) {
+            pref_h_resolution = 0;
+            pref_v_resolution = 0;
+            return;
+        }
+        int h_value = 0;
+        while ((option_length > 0) && (*option >= '0') && (*option <= '9')) {
+            h_value = h_value * 10 + (*option - '0');
+            option_length--;
+            option++;
+        }
+        if ((option_length < 2) || (*option != 'x')) return;
+        option_length--;
+        option++;
+        int v_value = 0;
+        while ((option_length > 0) && (*option >= '0') && (*option <= '9')) {
+            v_value = v_value * 10 + (*option - '0');
+            option_length--;
+            option++;
+        }
+        if (option_length != 0) return;
+        pref_h_resolution = h_value;
+        pref_v_resolution = v_value;
+        return;
+    }
+}
+
+static void parse_cmd_line(uintptr_t cmd_line_addr, int cmd_line_size)
+{
+    pref_h_resolution = UINT32_MAX;
+    pref_v_resolution = UINT32_MAX;
+    rotate = false;
+
+    if (cmd_line_addr != 0) {
+        const char *cmd_line = (const char *)cmd_line_addr;
+        const char *option = cmd_line;
+        int option_length = 0;
+        for (int i = 0; i < cmd_line_size; i++) {
+            switch (cmd_line[i]) {
+              case '\0':
+                parse_option(option, option_length);
+                return;
+              case ' ':
+                parse_option(option, option_length);
+                option = &cmd_line[i+1];
+                option_length = 0;
+                break;
+              default:
+                option_length++;
+                break;
+            }
+        }
+    }
+}
+
 static efi_status_t alloc_memory(void **ptr, size_t size, efi_phys_addr_t max_addr)
 {
     efi_status_t status;
@@ -300,11 +389,11 @@ static efi_graphics_output_t *find_gop(efi_handle_t *handles, size_t handles_siz
             continue;
         }
 
-#if DEBUG
-        print_string("Found GOP with ");
-        print_dec(mode->max_mode);
-        print_string(" modes\n");
-#endif
+        if (debug) {
+            print_string("Found GOP with ");
+            print_dec(mode->max_mode);
+            print_string(" modes\n");
+        }
 
         // Systems that use the UEFI Console Splitter may provide multiple GOP
         // devices, not all of which are backed by real hardware. The workaround
@@ -314,9 +403,9 @@ static efi_graphics_output_t *find_gop(efi_handle_t *handles, size_t handles_siz
         void *con_out = NULL;
         status = efi_call_bs(handle_protocol, handle, &EFI_CONSOLE_OUT_DEVICE_GUID, &con_out);
         if (status == EFI_SUCCESS) {
-#if DEBUG
-            print_string("This GOP implements the ConOut protocol\n");
-#endif
+            if (debug) {
+                print_string("This GOP implements the ConOut protocol\n");
+            }
             return gop;
         }
 
@@ -340,27 +429,61 @@ static efi_status_t set_screen_info_from_gop(screen_info_t *si, efi_handle_t *ha
 
     efi_gop_mode_t *mode = efi_table_attr(gop, mode);
 
+    bool use_current_mode = (pref_h_resolution == 0) && (pref_v_resolution == 0);
+
+    if (debug) {
+        print_string("Requested size : ");
+        if ((pref_h_resolution == UINT32_MAX) && (pref_v_resolution == UINT32_MAX)) {
+            print_string("auto");
+        } else {
+            print_dec(pref_h_resolution);
+            print_string(" x ");
+            print_dec(pref_v_resolution);
+        }
+        if (rotate) {
+            print_string(" rotated");
+        }
+        print_string("\n");
+    }
+
     efi_gop_mode_info_t best_info;
     best_info.h_resolution = UINT32_MAX;
     best_info.v_resolution = UINT32_MAX;
 
     uint32_t best_mode = UINT32_MAX;
-    for (uint32_t mode_num = 0; mode_num < mode->max_mode; mode_num++) {
-        efi_gop_mode_info_t *info;
-        uintn_t              info_size;
-        status = efi_call_proto(gop, query_mode, mode_num, &info_size, &info);
-        if (status != EFI_SUCCESS) {
-            continue;
+    if (use_current_mode) {
+        best_mode = mode->mode;
+        best_info = *mode->info;
+    } else {
+        for (uint32_t mode_num = 0; mode_num < mode->max_mode; mode_num++) {
+            efi_gop_mode_info_t *info;
+            uintn_t              info_size;
+            status = efi_call_proto(gop, query_mode, mode_num, &info_size, &info);
+            if (status != EFI_SUCCESS) {
+                continue;
+            }
+            if ((info->h_resolution == pref_h_resolution) && (info->v_resolution == pref_v_resolution)) {
+                best_mode = mode_num;
+                best_info = *info;
+                break;
+            }
+            if (rotate) {
+                if (info->v_resolution >= MIN_H_RESOLUTION
+                 && info->h_resolution >= MIN_V_RESOLUTION
+                 && info->v_resolution < best_info.v_resolution) {
+                    best_mode = mode_num;
+                    best_info = *info;
+                }
+            } else {
+                if (info->h_resolution >= MIN_H_RESOLUTION
+                 && info->v_resolution >= MIN_V_RESOLUTION
+                 && info->h_resolution < best_info.h_resolution) {
+                    best_mode = mode_num;
+                    best_info = *info;
+                }
+            }
+            efi_call_bs(free_pool, info);
         }
-
-        if (info->h_resolution >= MIN_H_RESOLUTION
-         && info->v_resolution >= MIN_V_RESOLUTION
-         && info->h_resolution < best_info.h_resolution) {
-            best_mode = mode_num;
-            best_info = *info;
-        }
-
-        efi_call_bs(free_pool, info);
     }
     if (best_mode == UINT32_MAX) {
         print_string("No suitable screen resolution found\n");
@@ -374,7 +497,7 @@ static efi_status_t set_screen_info_from_gop(screen_info_t *si, efi_handle_t *ha
     si->lfb_width  = best_info.h_resolution;
     si->lfb_height = best_info.v_resolution;
     si->lfb_base   = lfb_base;
-#ifdef __x86_64__
+#if (ARCH_BITS == 64)
     if (lfb_base >> 32) {
         si->capabilities |= LFB_CAPABILITY_64BIT_BASE;
         si->ext_lfb_base = lfb_base >> 32;
@@ -383,9 +506,9 @@ static efi_status_t set_screen_info_from_gop(screen_info_t *si, efi_handle_t *ha
 
     switch (best_info.pixel_format) {
       case PIXEL_RGB_RESERVED_8BIT_PER_COLOR:
-#if DEBUG
-        print_string("RGB32 mode\n");
-#endif
+        if (debug) {
+            print_string("RGB32 mode\n");
+        }
         si->lfb_depth       = 32;
         si->lfb_linelength  = best_info.pixels_per_scan_line * 4;
         si->red_size        = 8;
@@ -398,9 +521,9 @@ static efi_status_t set_screen_info_from_gop(screen_info_t *si, efi_handle_t *ha
         si->rsvd_pos        = 24;
         break;
       case PIXEL_BGR_RESERVED_8BIT_PER_COLOR:
-#if DEBUG
-        print_string("BGR32 mode\n");
-#endif
+        if (debug) {
+            print_string("BGR32 mode\n");
+        }
         si->lfb_depth       = 32;
         si->lfb_linelength  = best_info.pixels_per_scan_line * 4;
         si->red_size        = 8;
@@ -413,9 +536,9 @@ static efi_status_t set_screen_info_from_gop(screen_info_t *si, efi_handle_t *ha
         si->rsvd_pos        = 24;
         break;
       case PIXEL_BIT_MASK:
-#if DEBUG
-        print_string("Bit mask mode\n");
-#endif
+        if (debug) {
+            print_string("Bit mask mode\n");
+        }
         get_bit_range(best_info.pixel_info.red_mask,   &si->red_pos,   &si->red_size);
         get_bit_range(best_info.pixel_info.green_mask, &si->green_pos, &si->green_size);
         get_bit_range(best_info.pixel_info.blue_mask,  &si->blue_pos,  &si->blue_size);
@@ -424,9 +547,9 @@ static efi_status_t set_screen_info_from_gop(screen_info_t *si, efi_handle_t *ha
         si->lfb_linelength  = (best_info.pixels_per_scan_line * si->lfb_depth) / 8;
         break;
       default:
-#if DEBUG
-        print_string("Unsupported mode\n");
-#endif
+        if (debug) {
+            print_string("Unsupported mode\n");
+        }
         si->lfb_depth       = 4;
         si->lfb_linelength  = si->lfb_width / 2;
         si->red_size        = 0;
@@ -441,39 +564,41 @@ static efi_status_t set_screen_info_from_gop(screen_info_t *si, efi_handle_t *ha
     }
     si->lfb_size = si->lfb_linelength * si->lfb_height;
 
-#if DEBUG
-    print_string("FB base   : ");
-    print_hex((uintptr_t)lfb_base);
-    print_string("\n");
-    print_string("FB size   : ");
-    print_dec(si->lfb_width);
-    print_string(" x ");
-    print_dec(si->lfb_height);
-    print_string("\n");
-    print_string("FB format :");
-    print_string(" R"); print_dec(si->red_size);
-    print_string(" G"); print_dec(si->green_size);
-    print_string(" B"); print_dec(si->blue_size);
-    print_string(" A"); print_dec(si->rsvd_size);
-    print_string("\n");
-    print_string("FB stride : ");
-    print_dec(si->lfb_linelength);
-    print_string("\n");
-    print_string("Press any key to continue...\n");
-    wait_for_key();
-#endif
-
-    status = efi_call_proto(gop, set_mode, best_mode);
-    if (status != EFI_SUCCESS) {
-        print_string("Set GOP mode failed\n");
-        return status;
+    if (debug) {
+        print_string("FB base   : ");
+        print_hex((uintptr_t)lfb_base);
+        print_string("\n");
+        print_string("FB size   : ");
+        print_dec(si->lfb_width);
+        print_string(" x ");
+        print_dec(si->lfb_height);
+        print_string("\n");
+        print_string("FB format :");
+        print_string(" R"); print_dec(si->red_size);
+        print_string(" G"); print_dec(si->green_size);
+        print_string(" B"); print_dec(si->blue_size);
+        print_string(" A"); print_dec(si->rsvd_size);
+        print_string("\n");
+        print_string("FB stride : ");
+        print_dec(si->lfb_linelength);
+        print_string("\n");
+        print_string("Press any key to continue...\n");
+        wait_for_key();
     }
 
-#if DEBUG
-    test_frame_buffer(si);
-    print_string("Press any key to continue...\n");
-    wait_for_key();
-#endif
+    if (!use_current_mode) {
+        status = efi_call_proto(gop, set_mode, best_mode);
+        if (status != EFI_SUCCESS) {
+            print_string("Set GOP mode failed\n");
+            return status;
+        }
+    }
+
+    if (debug) {
+        test_frame_buffer(si);
+        print_string("Press any key to continue...\n");
+        wait_for_key();
+    }
 
     return EFI_SUCCESS;
 }
@@ -552,7 +677,7 @@ static efi_status_t set_efi_info_and_exit_boot_services(efi_handle_t handle, boo
         goto fail;
     }
 
-#ifdef __x86_64
+#if (ARCH_BITS == 64)
     boot_params->efi_info.loader_signature  = EFI64_LOADER_SIGNATURE;
 #else
     boot_params->efi_info.loader_signature  = EFI32_LOADER_SIGNATURE;
@@ -562,7 +687,7 @@ static efi_status_t set_efi_info_and_exit_boot_services(efi_handle_t handle, boo
     boot_params->efi_info.mem_desc_version  = mem_desc_version;
     boot_params->efi_info.mem_map           = (uintptr_t)mem_map;
     boot_params->efi_info.mem_map_size      = mem_map_size;
-#ifdef __x86_64__
+#if (ARCH_BITS == 64)
     boot_params->efi_info.sys_tab_hi        = (uintptr_t)sys_table >> 32;
     boot_params->efi_info.mem_map_hi        = (uintptr_t)mem_map   >> 32;
 #endif
@@ -574,7 +699,7 @@ fail:
 static void set_e820_map(boot_params_t *params)
 {
     uintptr_t mem_map_addr = params->efi_info.mem_map;
-#ifdef __x86_64__
+#if (ARCH_BITS == 64)
     mem_map_addr |= (uintptr_t)params->efi_info.mem_map_hi << 32;
 #endif
     size_t mem_map_size  = params->efi_info.mem_map_size;
@@ -659,6 +784,8 @@ boot_params_t *efi_setup(efi_handle_t handle, efi_system_table_t *sys_table_arg,
 
     boot_params->code32_start = (uintptr_t)startup32;
 
+    parse_cmd_line(boot_params->cmd_line_ptr, boot_params->cmd_line_size);
+
     status = set_screen_info(boot_params);
     if (status != EFI_SUCCESS) {
         print_string("set_screen_info() failed\n");
@@ -679,6 +806,10 @@ fail:
     print_string("efi_setup() failed\n");
 
     while (1) {
+#if defined(__x86_64) || defined(__i386__)
         __asm__("hlt");
+#elif defined(__loongarch_lp64)
+        __asm__("idle 0");
+#endif
     }
 }
