@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0
-// Copyright (C) 2004-2023 Sam Demeulemeester
+// Copyright (C) 2004-2025 Sam Demeulemeester
 //
 // ------------------------
 // This file is used to detect quirks on specific hardware
@@ -50,6 +50,16 @@ static void asus_tusl2_configure_mux(void)
     outb(0xAA, 0x2E);
 }
 
+static int *get_motherboard_cache(void)
+{
+    if (l2_cache == 0) {
+        return &l2_cache;
+    } else if (l3_cache == 0) {
+        return &l3_cache;
+    }
+    return NULL;
+}
+
 static void get_m1541_l2_cache_size(void)
 {
     if (l2_cache != 0) {
@@ -69,9 +79,27 @@ static void get_m1541_l2_cache_size(void)
     if (reg == 0b10) { l2_cache = 1024; }
 }
 
+static void get_vt82c597_mb_cache_size(void)
+{
+    int *const mb_cache = get_motherboard_cache();
+    if (!mb_cache) {
+        return;
+    }
+
+    // Check if cache is enabled with CC1 Register[7:6]
+    if ((pci_config_read8(0, 0, 0, 0x50) & 0xc0) != 0x80) {
+        return;
+    }
+
+    // Get cache size with CC2 Register[1:0]
+    const uint8_t reg = pci_config_read8(0, 0, 0, 0x51) & 0x03;
+
+    *mb_cache = 256 << reg;
+}
+
 static void disable_temp_reporting(void)
 {
-    enable_temperature = false;
+    enable_temp_cpu = false;
 }
 
 static void amd_k8_revfg_temp(void)
@@ -117,6 +145,36 @@ static void loongson_7a00_ehci_workaround(void)
     write8((uint8_t *)(reg_addr + 0x3830), 0x0);
 }
 
+
+static void unhide_ich_0_5(void)
+{
+    uint32_t smb_ctrl_reg;
+
+    smb_ctrl_reg = pci_config_read32(0, 0x1F, 0x00, 0xF0);
+    smb_ctrl_reg &= ~(1 << 19);
+    pci_config_write32(0, 0x1F, 0x00, 0xF0, smb_ctrl_reg);
+}
+
+static void unhide_asus_smbus(void)
+{
+    uint32_t smb_ctrl_reg;
+
+    smb_ctrl_reg = pci_config_read32(0, 0x1F, 0x00, 0xF0);
+    pci_config_write32(0, 0x1F, 0x00, 0xF0, smb_ctrl_reg | 0x01);
+
+    // Get MMIO base address
+    smb_ctrl_reg = pci_config_read32(0, 0x1F, 0x00, 0xF0);
+
+    if ((smb_ctrl_reg & 1) == 0) { return; }
+
+    // Read MMIO at offset 0x3418
+    uintptr_t base = (uintptr_t)(smb_ctrl_reg & ~0x3FFFu);
+    uint32_t *ptr = (uint32_t *)(base + 0x3418u);
+
+    // Set Bit 3 to 0 to enable SMBUS Controler
+    *ptr &= ~(1u << 3);
+}
+
 // ---------------------
 // -- Public function --
 // ---------------------
@@ -140,6 +198,16 @@ void quirks_init(void)
         quirk.process = get_m1541_l2_cache_size;
     }
 
+    //  -----------------------------------------------
+    //  -- VIA VP3 (VT82C597), MVP3 (VT82C598) Quirk --
+    //  -----------------------------------------------
+    // Motherboard cache detection
+    else if (quirk.root_vid == PCI_VID_VIA && (quirk.root_did == 0x0597 || quirk.root_did == 0x0598)) { // VIA VT82C597/8
+        quirk.id    = QUIRK_VIA_VP3;
+        quirk.type |= QUIRK_TYPE_MEM_SIZE;
+        quirk.process = get_vt82c597_mb_cache_size;
+    }
+
     //  ------------------------
     //  -- ASUS TUSL2-C Quirk --
     //  ------------------------
@@ -221,9 +289,48 @@ void quirks_init(void)
     //  -----------------------------------------------------------
     //  -- Loongson 7A1000 and 7A2000 chipset USB 2.0 workaround --
     //  -----------------------------------------------------------
-    if (quirk.root_vid == PCI_VID_LOONGSON && quirk.root_did == 0x7a00) {
-        quirk.id    = QUIRK_LOONGSON7A00_EHCI_WORKARD;
-        quirk.type |= QUIRK_TYPE_USB;
-        quirk.process = loongson_7a00_ehci_workaround;
+    if (quirk.root_vid == PCI_VID_LOONGSON) {
+        if (pci_config_read16(0, 4, 1, PCI_DID_REG) == 0x7a14) {
+            quirk.id    = QUIRK_LOONGSON7A00_EHCI_WORKARD;
+            quirk.type |= QUIRK_TYPE_USB;
+            quirk.process = loongson_7a00_ehci_workaround;
+        }
+    }
+
+    //  -----------------------------------------------------
+    //  -- Unhide SMBus on early Intel Southbridges (ICHx) --
+    //  -----------------------------------------------------
+    if (quirk.root_vid == PCI_VID_INTEL) {
+
+        uint16_t sb_lpc_did = pci_config_read16(0, 0x1F, 0, PCI_DID_REG);
+        uint16_t root_subvid  = pci_config_read16(0, 0, 0, PCI_SUB_VID_REG);
+        uint16_t root_subdid  = pci_config_read16(0, 0, 0, PCI_SUB_DID_REG);
+
+        switch(sb_lpc_did)
+        {
+            case PCI_DID_LPC_ICH:
+            case PCI_DID_LPC_ICH1:
+            case PCI_DID_LPC_ICH2:
+            case PCI_DID_LPC_ICH2M:
+            case PCI_DID_LPC_ICH3:
+            case PCI_DID_LPC_ICH3M:
+            case PCI_DID_LPC_ICH4:
+            case PCI_DID_LPC_ICH4M:
+            case PCI_DID_LPC_ICH5:
+            case PCI_DID_LPC_6300ESB:
+                quirk.id      = QUIRK_UNHIDE_ICH05;
+                quirk.type   |= QUIRK_TYPE_SMBUS;
+                quirk.process = unhide_ich_0_5;
+                break;
+            case PCI_DID_LPC_ICH6M:
+                if (root_subvid == PCI_VID_DELL && root_subdid == 0x0182) { // Dell D620
+                    quirk.id      = QUIRK_UNHIDE_ASUS_SMBUS;
+                    quirk.type   |= QUIRK_TYPE_SMBUS;
+                    quirk.process = unhide_asus_smbus;
+                }
+                break;
+            default:
+                break;
+        }
     }
 }
