@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0
-// Copyright (C) 2004-2023 Sam Demeulemeester
+// Copyright (C) 2004-2024 Sam Demeulemeester
 #ifndef _IMC_H_
 #define _IMC_H_
 
@@ -25,6 +25,12 @@ void get_imc_config_intel_icl(void);
 /* Memory configuration Detection for Intel Alder Lake */
 void get_imc_config_intel_adl(void);
 
+/* Memory configuration Detection for Intel Metor Lake */
+void get_imc_config_intel_mtl(void);
+
+/* Memory configuration Detection for Loongson LoongArch DDR4 CPU family */
+void get_imc_config_loongson_ddr4(void);
+
 /**
  * ECC Polling Code for various IMCs
  */
