@@ -23,9 +23,9 @@ float cpu_temp_offset = 0;
 // Public Functions
 //------------------------------------------------------------------------------
 
-void temperature_init(void)
+void cpu_temp_init(void)
 {
-    if (!enable_temperature) {
+    if (!enable_temp_cpu) {
         return;
     }
 
@@ -35,7 +35,12 @@ void temperature_init(void)
     }
 }
 
-int get_cpu_temperature(void)
+int get_cpu_temp(void)
 {
     return (int)(__iocsrrd_w(0x428));
 }
+
+int get_ram_temp(uint8_t slot)
+{
+    return 0;
+}
