@@ -117,10 +117,11 @@ static usb_endpoint_desc_t *find_hub_endpoint_descriptor(const uint8_t *desc_buf
 
         if (header->type == USB_DESC_ENDPOINT && header->length == sizeof(usb_endpoint_desc_t)) {
             usb_endpoint_desc_t *endpoint = (usb_endpoint_desc_t *)curr_ptr;
-#if 0
-            print_usb_info("endpoint addr 0x%02x attr 0x%02x",
-                           (uintptr_t)endpoint->address, (uintptr_t)endpoint->attributes);
-#endif
+            if (usb_init_options & USB_DEBUG_HUB) {
+                print_usb_info("endpoint addr 0x%02x attr 0x%02x",
+                               (uintptr_t)endpoint->address, (uintptr_t)endpoint->attributes);
+                sleep(1);
+            }
             if ((endpoint->address & 0x80) && (endpoint->attributes & 0x3) == 0x3) {
                 return endpoint;
             }
@@ -149,6 +150,7 @@ static bool build_hub_info(const usb_hcd_t *hcd, const usb_hub_t *parent, int po
     hub->tt_think_time  = hub_desc.characteristics & 0x0060 >> 5;
     hub->power_up_delay = hub_desc.power_up_delay;
     hub->hs_parent      = usb_hs_parent(parent, port_num, ep0->device_speed);
+    hub->quirks         = USB_HUB_NO_QUIRKS;
 
     usb_endpoint_desc_t *ep1_desc = find_hub_endpoint_descriptor(hcd->ws->data_buffer, hcd->ws->data_length);
     if (ep1_desc == NULL) {
@@ -166,6 +168,14 @@ static bool build_hub_info(const usb_hcd_t *hcd, const usb_hub_t *parent, int po
     return true;
 }
 
+static void add_hub_quirks(const usb_device_desc_t *device, usb_hub_t *hub)
+{
+    if ((device->vendor_id == USB_VID_AMERICAN_MEGATRENDS) && (device->product_id == 0xff01)) {
+        // add quirk for AMI Virtual Hub - see issue #523
+        hub->quirks |= USB_HUB_DONT_DISABLE_PORTS;
+    }
+}
+
 static bool get_hub_port_status(const usb_hcd_t *hcd, const usb_hub_t *hub, int port_num, uint32_t *port_status)
 {
     usb_setup_pkt_t setup_pkt;
@@ -223,10 +233,11 @@ static void get_keyboard_info_from_descriptors(const uint8_t *desc_buffer, int d
 
         if (header->type == USB_DESC_INTERFACE && header->length == sizeof(usb_interface_desc_t)) {
             const usb_interface_desc_t *ifc = (const usb_interface_desc_t *)curr_ptr;
-#if 0
-            print_usb_info("interface %i class %i subclass %i protocol %i",
-                           ifc->interface_num, ifc->class, ifc->subclass, ifc->protocol);
-#endif
+            if (usb_init_options & USB_DEBUG_KBD) {
+                print_usb_info("interface %i class %i subclass %i protocol %i",
+                               ifc->interface_num, ifc->class, ifc->subclass, ifc->protocol);
+                sleep(1);
+            }
             if (ifc->class == 3 && ifc->subclass == 1 && ifc->protocol == 1) {
                 kbd = &keyboards[*num_keyboards];
                 kbd->interface_num = ifc->interface_num;
@@ -235,10 +246,11 @@ static void get_keyboard_info_from_descriptors(const uint8_t *desc_buffer, int d
             }
         } else if (header->type == USB_DESC_ENDPOINT && header->length == sizeof(usb_endpoint_desc_t)) {
             usb_endpoint_desc_t *endpoint = (usb_endpoint_desc_t *)curr_ptr;
-#if 0
-            print_usb_info("endpoint addr 0x%02x attr 0x%02x",
-                           (uintptr_t)endpoint->address, (uintptr_t)endpoint->attributes);
-#endif
+            if (usb_init_options & USB_DEBUG_KBD) {
+                print_usb_info("endpoint addr 0x%02x attr 0x%02x",
+                               (uintptr_t)endpoint->address, (uintptr_t)endpoint->attributes);
+                sleep(1);
+            }
             if (kbd && (endpoint->address & 0x80) && (endpoint->attributes & 0x3) == 0x3) {
                 kbd->endpoint_num    = endpoint->address & 0xf;
                 kbd->max_packet_size = endpoint->max_packet_size;
@@ -349,6 +361,8 @@ static bool scan_hub_ports(const usb_hcd_t *hcd, const usb_hub_t *hub, int *num_
             continue;
         }
 
+        if (hub->quirks & USB_HUB_DONT_DISABLE_PORTS) continue;
+
         // If we didn't find any keyboards, we can disable the port and release the slot.
         build_setup_packet(&setup_pkt, USB_REQ_TO_HUB_PORT | USB_REQ_CLASS, HUB_CLR_FEATURE, HUB_PORT_ENABLE, port_num, 0);
         (void)hcd->methods->setup_request(hcd, hub->ep0, &setup_pkt);
@@ -751,6 +765,7 @@ bool find_attached_usb_keyboards(const usb_hcd_t *hcd, const usb_hub_t *hub, int
         if (!build_hub_info(hcd, hub, port_num, &ep0, &new_hub, &ep1)) {
             return false;
         }
+        add_hub_quirks(device, &new_hub);
         if (!configure_device(hcd, &ep0, config_num)) {
             return false;
         }
